<?php 

// regular expression syntax-highlighter from http://stevenlevithan.com/regex/syntaxhighlighter/
// dynamic regular expression syntax-highlighter from http://github.com/jmrware/DynamicRegexHighlighter/archives/master
// extension author: balrok  http://balrok.com

class EReSyntaxHighlighter extends CWidget 
{ 
	public $cssFile;
	// which class should be highlighted
	public $className = 'regex';
	// do you want the dynamic highlighter too?
	public $dynamic = true;

	// if you want to wrap the code in a function-name calling it twice will toggle
	// this won't autoload your stuff it has to be toggled on via this function
	public $functionName;

	private $baseUrl;
	private $clientScript;

    /** 
     * Initialize the widget 
     */ 
    public function init() 
    { 
		$this->baseUrl = Yii::app()->getAssetManager()->publish(dirname(__FILE__).DIRECTORY_SEPARATOR.'assets');

		$this->clientScript = Yii::app()->getClientScript();
		// jsresyntaxhighlighter.css  jsresyntaxhighlighter.js  jsresyntaxhighlighter-min.js

		if ($this->dynamic)
			$this->clientScript->registerScriptFile($this->baseUrl.'/DynamicRegexHighlighter.js');

		$this->clientScript->registerScriptFile($this->baseUrl.'/jsresyntaxhighlighter-min.js');

		if (empty($this->cssFile))
			$this->cssFile = $this->baseUrl.'/jsresyntaxhighlighter.css';
		$this->clientScript->registerCssFile($this->cssFile);
    } 

 	/** 
     * Executes the widget 
     */ 
    public function run() 
    { 
		if (!empty($this->functionName)) // then don't autoload dynamic
		{
        	$this->clientScript->registerScript(__CLASS__, 'window.reAutoLoad = false;', CClientScript::POS_HEAD);
        	$this->clientScript->registerScript(__CLASS__, $this->getWrappedInitScripts(), CClientScript::POS_END);
		}
		else
        	$this->clientScript->registerScript(__CLASS__, $this->getInitScripts(), CClientScript::POS_END);
    }

	private function getWrappedInitScripts()
	{
		// this is the initfunction of the highlighter - if a functionname was specified we have to add my hacky code here
		// which wraps around the init script
		$scripts = '';
		$scripts .= '
		// displayRegex will toggle what the next functioncall will do
		var displayRegex = true;
		// reDataStore will cache our nonmarked-up regular expressions
		var reDataStore = new Array();
		function '.$this->functionName.'(){
			if (!displayRegex) {
				var regexEls = document.getElementsByClassName("'.$this->className.'"),
				len = regexEls.length;
				for (i = 0; i < len; i++) regexEls[i].innerHTML = reDataStore[i];
				displayRegex = true;
				return;
			} else {
				var regexEls = document.getElementsByClassName("'.$this->className.'"),
				len = regexEls.length;
				for (i = 0; i < len; i++) reDataStore[i] = regexEls[i].innerHTML;
			}
			displayRegex = false;
			';
		$scripts .= $this->getInitScripts();
		$scripts .= '}';
		return $scripts;
	}
 
    private function getInitScripts() 
    { 
		$scripts = '
				var regexEls = document.getElementsByClassName("'.$this->className.'"),
					len = regexEls.length,
					el, text, i;

				for (i = 0; i < len; i++) {
					el = regexEls[i];
					text = el.textContent || el.innerText;

					// this is where the magic happens
					el.innerHTML = highlightJsReSyntax(text);';
		// if we want to toggle it by button don't autoload it
		if ($this->dynamic && $this->functionName)
			$scripts.='reHighlightElement(el);';
		$scripts .='}';


		return $scripts;
    }
} 
?> 
